@@ -5,10 +5,12 @@ use strict;
 use FlashVideo::Utils;
 use FlashVideo::JSON;
 use HTML::Entities;
+use HTML::TreeBuilder;
+use HTML::Element;
 use Encode;
 use Data::Dumper;
 
-our $VERSION = '0.09.01';
+our $VERSION = '0.09.03';
 sub Version() { $VERSION;}
 
 sub extract_attributes {
@@ -17,7 +19,6 @@ sub extract_attributes {
   my %attrib;
   while ($substr =~ m/([-a-z0-9]+)\s*=\"([^\"]+)\"/gi) {
     $attrib{$1} = $2;
-#    info "adding {".$1."}=".$2;
   }
   return \%attrib;
 }
@@ -40,7 +41,25 @@ sub find_video {
 
 
   my $productionid;
-  if ( $id )
+  ($productionid) = $browser->content =~ /\"productionId\":\"([^\"]+)\"/i;
+  if (! $productionid) {
+    ($productionid) = $browser->content =~ / data-video-id\s*=\s*\"([^\"]+)\"/i;
+  }
+  $productionid =~ s%^.*/%%;
+  $productionid =~ tr%_\.%/#%;
+  debug "Production ID $productionid\n";
+  die "No id (filter) found in URL or production id\n" unless $productionid;
+
+  my $hls_dl= 1;
+  my $rtmp_dl = 1;
+  if ($prefs->{type} =~ /hls/) {
+    $rtmp_dl = 0;
+  }
+  if ($prefs->{type} =~ /rtmp/) {
+    $hls_dl = 0;
+  }
+
+  if ( $id && $rtmp_dl)
   {
 
     $browser->post("http://mercury.itv.com/PlaylistService.svc",
@@ -76,16 +95,9 @@ sub find_video {
 </SOAP-ENV:Envelope>
 EOF
 
+    debug $browser->content;
   }
-  else {
-    ($productionid) = $browser->content =~ /\"productionId\":\"([^\"]+)\"/i;
-    if (! $productionid) {
-      ($productionid) = $browser->content =~ / data-video-id\s*=\s*\"([^\"]+)\"/i;
-    }
-    $productionid =~ s%^.*/%%;
-    $productionid =~ tr%_\.%/#%;
-    debug "Production ID $productionid\n";
-    die "No id (filter) found in URL or production id\n" unless $productionid;
+  elsif ($rtmp_dl) {
     $browser->post($itv_params->{'data-playlist-url'},
       Content_Type => "text/xml; charset=utf-8",
       Referer      => "http://www.itv.com/mercury/Mercury_VideoPlayer.swf?v=1.5.309/[[DYNAMIC]]/2",
@@ -133,20 +145,18 @@ EOF
 </soapenv:Envelope>
 EOF
 
+    debug $browser->content;
   }
   # We want the RTMP url within a <Video timecode=...> </Video> section.
-  debug $browser->content;
 
-  if ( $browser->content =~ m%<faultcode>InvalidEntity</faultcode>% ) {
 
-#debuging
-$browser->add_handler("request_send", sub { shift->dump; return });
-$browser->add_handler("response_done", sub { shift->dump; return });
+  if ( $hls_dl && (! $rtmp_dl || ($rtmp_dl && 
+        $browser->content =~ m%<faultcode>InvalidEntity</faultcode>%) ) ) {
 
-    info "Trying IOS download...";
-    info "Title: $og_title";
-    info "Episode Title ".$itv_params->{'data-video-episode'};
-    info "Series: ".$itv_params->{'data-video-title'};
+    debug "Trying IOS download...";
+    debug "Title: $og_title";
+    debug "Episode Title ".$itv_params->{'data-video-episode'};
+    debug "Series: ".$itv_params->{'data-video-title'};
 
     my $ios_playlist_url = $itv_params->{'data-video-playlist'};
     my $hmac = $itv_params->{'data-video-hmac'};
@@ -155,8 +165,8 @@ $browser->add_handler("response_done", sub { shift->dump; return });
         $ios_playlist_url = $itv_params->{'data-video-id'};
     }
 
-    info "ios url: $ios_playlist_url";
-    info "hamc: $hmac";
+    debug "ios url: $ios_playlist_url";
+    debug "hamc: $hmac";
     my $hls_m3u = 'https://127.0.0.1/test.m3u';
     
     $browser->agent('Mozilla/5.0 (X11; Linux x86_64; rv:10.0) Gecko/20150101 Firefox/47.0 (Chrome)');
@@ -173,7 +183,6 @@ $browser->add_handler("response_done", sub { shift->dump; return });
     my $playlist = from_json($browser->content);
     my $video_id = $playlist->{Playlist}->{Video};
     my $base_url = $video_id->{Base};
-    info "base:". $base_url;
     my @media_files = @{$video_id->{MediaFiles}};
 
     debug Data::Dumper::Dumper(@media_files);
@@ -184,7 +193,7 @@ $browser->add_handler("response_done", sub { shift->dump; return });
     my $file_id = $productionid;
     $file_id =~ tr%_/#%---%;
     my $filename = title_to_filename("$file_id\_$og_title\_hls", "mp4");
-    info "filename: $filename";
+    debug "filename: $filename";
 
     $browser->cookie_jar( {} ); # keep cookies
     $browser->add_header( Referer => undef);
@@ -384,5 +393,76 @@ sub itv_swfhash_data {
     swfUrl  => $url;
 }
 
+sub search {
+  my ($self, $search) = @_;
+
+  my $browser = FlashVideo::Mechanize->new;
+  my @links;
+  my %processed = {};
+
+  $browser->allow_redirects;
+  $search  =~ s/\\s/-/g;
+  $search =~ tr/ /-/;
+  $search =~ s/---*/-/g;
+  $search - lc($search);
+
+  $browser->get('https://www.itv.com/hub/shows');
+  
+  my $page = $browser->content;
+  my $series;
+
+  while ($page =~ m%<a href="(https://www.itv.com/hub/[^"]*)"[^>]* data-content-type="programme"[^>]*>%g) {
+    my $showurl = $1;
+    next if defined $processed{$showurl};
+    if ($showurl =~ m/$search/i) {
+      $browser->get($showurl);
+      my $progpage = $browser->content;
+      my $root = HTML::TreeBuilder->new;
+      # must set otherwise time and section tags are ignored
+      # html5 tags are missing.
+      $root->ignore_unknown(0);
+      $root->parse($progpage);
+       
+      my $episodes = $root->look_down(_tag => 'h2', class => 'episode-info__episode-count');
+      info  $episodes->as_text;
+
+      my $prog_title = $root->look_down(_tag => 'h1', class => 'episode-info__programme-title');
+
+      my $episodes = $root->look_down(_tag => 'div', 'id' => 'more-episodes');
+      my @sections = $episodes->look_down(_tag => 'section', class => 'module module--secondary js-series-group');
+      foreach my $section (@sections) {
+#        info $section->attr_get_i('id');
+        my $h2a = $section->look_down(_tag => 'h2', class => 'module__heading');
+        $series = $h2a->as_text;
+        my $li = $section->find('li');
+        my $chan = $li->attr('class');
+        $chan =~ s%^.*--([a-z0-9]+)-pos .*$%$1%;
+
+        my @progs = $section->look_down(_tag => 'a', 'data-content-type' => 'episode');
+
+        foreach my $prog (@progs) {
+          my $url = $prog->attr_get_i('href');
+          my $div = $prog->look_down(_tag => 'div', class => 'tout__body media__body');
+
+          my $h3 = $div->find('h3');
+          my $trans = $div->find('time');
+          my $d = $trans->as_text;
+          $d =~ s/^\s*\w+//;
+          $d =~ s/\s*$//;
+          my $name = $prog_title->as_text . " - $series episode" . $h3->as_text . " - $d ($chan)";
+          my $desc = $div->find('p');
+          my $episode = { name => $name,
+                          url => $url,
+                          description => $desc->as_text};
+
+          push @links, $episode;
+        }
+      }
+    }
+    $processed{$showurl} = $showurl;
+  }
+
+  return @links;
+}
 
 1;
